package main

import (
	"encoding/json"
	"io/ioutil"
	"log"
	"mmvs/swagger"
	"net/http"
)

func matchmakingHandler(w http.ResponseWriter, r *http.Request) {
	if r.Method != http.MethodPost {
		http.Error(w, "Method not allowed", http.StatusMethodNotAllowed)
		return
	}

	// Read HTTP request body
	body, err := ioutil.ReadAll(r.Body)
	if err != nil {
		http.Error(w, "Failed to read request body", http.StatusBadRequest)
		return
	}
	defer r.Body.Close()

	// Parse HTTP Body into business struct
	var req swagger.MatchmakingPostProcessingReq
	err = json.Unmarshal(body, &req)
	if err != nil {
		log.Println(string(body))
		http.Error(w, "Failed to unmarshal request body", http.StatusBadRequest)
		return
	}

	// Construct post-processing logic

	// 1. Continuously requeue all tickets until they time out
	// -------------------------------------------------
	// rsp := swagger.MatchmakingPostProcessingRsp{
	// 	RequeuedTickets: req.Data.Battle.Tickets,
	// }

	// 2. Divide each ticket from the original battle to form new, separate battles
	// -------------------------------------------------
	// battles := make([]swagger.BattleProposal, 0, len(req.Data.Battle.Tickets))
	// for i := range req.Data.Battle.Tickets {
	// 	battles = append(battles, swagger.BattleProposal{
	// 		Tickets: []swagger.MmTicketEntityPublic{req.Data.Battle.Tickets[i]},
	// 	})
	// }
	// rsp := swagger.MatchmakingPostProcessingRsp{
	// 	Battles: battles,
	// }

	// 3. Maintain the original battle configuration without any changes
	// -------------------------------------------------
	rsp := swagger.MatchmakingPostProcessingRsp{
		Battles: []swagger.BattleProposal{
			*req.Data.Battle,
		},
	}

	// Marshal business struct
	rspJSON, err := json.Marshal(rsp)
	if err != nil {
		http.Error(w, "Failed to marshal response", http.StatusInternalServerError)
		return
	}

	// Write HTTP response header and body
	w.Header().Set("Content-Type", "application/json")
	w.Write(rspJSON)

	// Print necessary logs
	getTicketIDs := func(tickets []swagger.MmTicketEntityPublic) []string {
		ticketIDs := make([]string, 0, len(tickets))
		for i := range tickets {
			ticketIDs = append(ticketIDs, tickets[i].Id)
		}
		return ticketIDs
	}
	log.Printf("[DEBUG] Req tickets: %v", getTicketIDs(req.Data.Battle.Tickets))
	for i := range rsp.Battles {
		log.Printf("[DEBUG] Rsp sub-%d-battle: %v", i, getTicketIDs(rsp.Battles[i].Tickets))
	}
	log.Printf("[DEBUG] Rsp dropped tickets: %v", getTicketIDs(rsp.RequeuedTickets))
}

func main() {
	http.HandleFunc("/pgos_matchmaking", matchmakingHandler)
	log.Fatal(http.ListenAndServe(":8080", nil))
}
